/**
 * Character range from start to end (inclusive).
 */

export const any = {
  type: 'characterClass',
  escape: '.',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
export const digit = {
  type: 'characterClass',
  escape: '\\d',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
export const nonDigit = {
  type: 'characterClass',
  escape: '\\D',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
export const word = {
  type: 'characterClass',
  escape: '\\w',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
export const nonWord = {
  type: 'characterClass',
  escape: '\\W',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
export const whitespace = {
  type: 'characterClass',
  escape: '\\s',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
export const nonWhitespace = {
  type: 'characterClass',
  escape: '\\S',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};

/**
 * @deprecated Renamed to `nonDigit`.
 */
export const notDigit = nonDigit;

/**
 * @deprecated Renamed to `nonWord`.
 */
export const notWord = nonWord;

/**
 * @deprecated Renamed to `nonWhitespace`.
 */
export const notWhitespace = nonWhitespace;
export function charClass(...elements) {
  if (elements.some(e => e.isNegated)) {
    throw new Error('`charClass` should receive only non-negated character classes');
  }
  if (elements.length === 1) {
    return elements[0];
  }
  return {
    type: 'characterClass',
    chars: elements.map(c => getAllChars(c)).flat(),
    ranges: elements.map(c => c.ranges).flat(),
    isNegated: false,
    encode: encodeCharacterClass
  };
}
export function charRange(start, end) {
  if (start.length !== 1) {
    throw new Error('`charRange` should receive only single character `start` string');
  }
  if (end.length !== 1) {
    throw new Error('`charRange` should receive only single character `end` string');
  }
  if (start > end) {
    throw new Error('`start` should be before or equal to `end`');
  }
  return {
    type: 'characterClass',
    chars: [],
    ranges: [{
      start,
      end
    }],
    isNegated: false,
    encode: encodeCharacterClass
  };
}
export function anyOf(characters) {
  const chars = characters.split('').map(c => escapeForCharacterClass(c));
  if (chars.length === 0) {
    throw new Error('`anyOf` should received at least one character');
  }
  return {
    type: 'characterClass',
    chars,
    ranges: [],
    isNegated: false,
    encode: encodeCharacterClass
  };
}
export function negated(element) {
  return {
    type: 'characterClass',
    chars: element.chars,
    ranges: element.ranges,
    isNegated: !element.isNegated,
    encode: encodeCharacterClass
  };
}

/**
 * @deprecated Renamed to `negated`.
 */
export const inverted = negated;
function encodeCharacterClass() {
  if (this.escape === undefined && this.chars.length === 0 && this.ranges.length === 0) {
    throw new Error('Character class should contain at least one character or character range');
  }

  // Direct rendering for single-character class
  if (this.escape !== undefined && !this.chars.length && !this.ranges.length && !this.isNegated) {
    return {
      precedence: 'atom',
      pattern: this.escape
    };
  }
  const allChars = getAllChars(this);

  // If passed characters includes hyphen (`-`) it need to be moved to
  // first (or last) place in order to treat it as hyphen character and not a range.
  // See: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_expressions/Character_classes#types
  const hyphen = allChars.includes('-') ? '-' : '';
  const caret = allChars.includes('^') ? '^' : '';
  const otherChars = allChars.filter(c => c !== '-' && c !== '^').join('');
  const ranges = this.ranges.map(({
    start,
    end
  }) => `${start}-${end}`).join('');
  const negation = this.isNegated ? '^' : '';
  let pattern = `[${negation}${ranges}${otherChars}${caret}${hyphen}]`;
  if (pattern === '[^-]') pattern = '[\\^-]';
  return {
    precedence: 'atom',
    pattern
  };
}
function escapeForCharacterClass(text) {
  return text.replace(/[\]\\]/g, '\\$&'); // $& means the whole matched string
}
function getAllChars(characterClass) {
  if (characterClass.escape === undefined) {
    return characterClass.chars;
  }
  return [characterClass.escape, ...characterClass.chars];
}
//# sourceMappingURL=character-class.js.map